package com.bpcl.lpg.core.payment.billdesk.impl;

import org.slf4j.LoggerFactory;
import org.apache.http.util.EntityUtils;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.impl.client.CloseableHttpClient;
import java.io.IOException;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.HttpEntity;
import java.util.List;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.NameValuePair;
import java.util.ArrayList;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.config.SocketConfig;
import org.apache.http.client.config.RequestConfig;
import java.text.DateFormat;
import java.util.Date;
import java.util.TimeZone;
import java.text.SimpleDateFormat;
import org.apache.commons.lang3.StringUtils;
import com.bpcl.lpg.core.model.EasysecsModel;
import java.util.Objects;
import com.bpcl.lpg.core.credentials.LPGCredentialsService;
import com.bpcl.lpg.core.payment.billdesk.LPGChecksumService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.slf4j.Logger;
import com.bpcl.lpg.core.payment.billdesk.LPGPaymentStatusCheckService;

public class LPGPaymentStatusCheckServiceImpl implements LPGPaymentStatusCheckService
{
    private static final Logger LOG;
    private static final int TIMEOUT = 30000;
    private ConfigurationService configurationService;
    private LPGChecksumService lpgChecksumService;
    private LPGCredentialsService credentialsService;
    
    public String getPaymentStatusFromBillDeskByTxnID(final String transactionID) {
        final String paymentQueryUrl = this.configurationService.getConfiguration().getString("billdesk.query.url");
        final EasysecsModel uidEncryptionKey = this.credentialsService.getSecretsByKey("billdeskMerchantId");
        final String merchantId = uidEncryptionKey.getSecret();
        Objects.requireNonNull(paymentQueryUrl, "Billdesk status check url should not be null");
        Objects.requireNonNull(merchantId, "Merchant id should not be null");
        final String date = this.getFormattedCurrentDate();
        final String msg = this.prepareMsg(transactionID, merchantId, date);
        final String response = this.getPaymentStatusFromBillDesk(paymentQueryUrl, msg);
        if (LPGPaymentStatusCheckServiceImpl.LOG.isInfoEnabled()) {
            LPGPaymentStatusCheckServiceImpl.LOG.info("PaymentStatusCheck for transactionId : {} msg: {} and response : {}", transactionID, msg, response);
        }
        return response;
    }
    
    private String prepareMsg(final String transactionID, final String merchantId, final String date) {
        final String[] msg = { "0122", merchantId, transactionID, date };
        final String msgWithPipe = StringUtils.join((Object[])msg, "|");
        final String checksumValue = this.lpgChecksumService.getChecksumValue(msgWithPipe);
        return msgWithPipe + "|" + checksumValue;
    }
    
    private String getFormattedCurrentDate() {
        final DateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
        simpleDateFormat.setTimeZone(TimeZone.getTimeZone("IST"));
        return simpleDateFormat.format(new Date());
    }
    
    private String getPaymentStatusFromBillDesk(final String queryUrl, final String msg) {
        final RequestConfig requestConfig = RequestConfig.custom().setConnectTimeout(30000).setSocketTimeout(30000).build();
        final SocketConfig socketConfig = SocketConfig.custom().setSoTimeout(30000).build();
        final CloseableHttpClient client = HttpClients.custom().setDefaultRequestConfig(requestConfig).setDefaultSocketConfig(socketConfig).build();
        final HttpPost post = new HttpPost(queryUrl);
        final List<NameValuePair> arguments = new ArrayList<NameValuePair>();
        arguments.add(new BasicNameValuePair("msg", msg));
        try {
            post.setEntity(new UrlEncodedFormEntity(arguments));
            final CloseableHttpResponse response = client.execute((HttpUriRequest)post);
            return this.getResponseMsg(response);
        }
        catch (final IOException exception) {
            LPGPaymentStatusCheckServiceImpl.LOG.error("Exception while executing the query status check controller", exception);
            return "";
        }
    }
    
    private String getResponseMsg(final CloseableHttpResponse response) throws IOException {
        final String body = (response == null) ? null : EntityUtils.toString(response.getEntity());
        return body;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public void setLpgChecksumService(final LPGChecksumService lpgChecksumService) {
        this.lpgChecksumService = lpgChecksumService;
    }
    
    public void setCredentialsService(final LPGCredentialsService credentialsService) {
        this.credentialsService = credentialsService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(LPGPaymentStatusCheckServiceImpl.class);
    }
}
